'**************************************************************************************************
'	NTFS Stuff v1.3: a quick and easy tool that could help us to locate those "significant" 
'	folders that, in some way, affect the normal application of NTFS permissions based on the
'	"inheritance" concept.
'	Authors: Mirko Iodice (mirko at notageek.it), Luca Alberti
'	Credits to:
'	- kris at irony.it for Javascripts
'	- Helios Ciancio for Windows 2008 UAC elevation bug fix
'	- Microsoft for constants and DACL related functions
'**************************************************************************************************

'---- Header Information --------------------------------------------
Dim ObjShell, objShellApp, ObjFSO, FSO, objWMI, objWMIService, objProc, objOutParams, objSecDescriptor, objOwner, ObjLog, objFolder, ColProcs, colDrives, ObjDrive, ObjExec, ObjGetFolder, objlocator, objservice, objfilesecsetting
Dim arrayChildACLs(), arrayParentACLs()
Dim x, ScriptEngine, strFirstArg, boolCommandLine, boolV, details_style, FolderbyUser, RootDriveScan, printParent, tempInheritance, ScriptName, logfile, strComputer, divID, nfolders, Sleeptime, Verbositylevel, Verbosity, RecursionLevel, ScriptFile,  CurrentPath, PreviousSlashes, Recursion, RootSlashes, i
Const ForReading = 1, ForWriting = 2, ForAppending = 8	'constants used for opening the output file

'---- Reference Information -----------------------------------------

ScriptName = "NTFS Stuff v1.3"	'Script Name
strComputer= "."	'local system
divID = 0	'html divs counter to open/close div tags
nfolders = 0	'used to enumerate the total folders
Sleeptime = 150	'default GoSubFolders sleep time value
Verbositylevel= 0	'default VerbosityLevel
RecursionLevel = 0	'default RecursionLevel
ScriptFile = WScript.ScriptFullName	'get script full file name
CurrentPath = Left(ScriptFile, InstrRev(ScriptFile, "\")-1)	'get the current directory
ScriptEngine = LCase(Right(WScript.FullName, 11))
strFirstArg = ""
boolV = False
logfile = CurrentPath & "\ntfs_stuffed.html"          'name of the output file

'---- Main objects --------------------------------------------------

Set ObjFSO = CreateObject("Scripting.FileSystemObject")
Set FSO = CreateObject("Scripting.FileSystemObject")
Set objLocator = CreateObject("WbemScripting.SWbemLocator")
Set objService = objLocator.ConnectServer (".", "root/cimv2")
Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
Set objShell = CreateObject("WScript.Shell")
Set objShellApp = CreateObject("Shell.Application")

'---- ControlFlags --------------------------------------------------
Const ALLOW_INHERIT = 33796
Const DENY_INHERIT = 37892
Const SE_OWNER_DEFAULTED = 1
Const SE_GROUP_DEFAULTED = 2
Const SE_DACL_PRESENT = 4
Const SE_DACL_DEFAULTED = 8
Const SE_SACL_PRESENT = 16
Const SE_SACL_DEFAULTED = 32
Const SE_DACL_AUTO_INHERIT_REQ = 256
Const SE_SACL_AUTO_INHERIT_REQ = 512
Const SE_DACL_AUTO_INHERITED = 1024
Const SE_SACL_AUTO_INHERITED = 2048
Const SE_DACL_PROTECTED = 4096
Const SE_SACL_PROTECTED = 8192
Const SE_SELF_RELATIVE = 32768

'---- ACE Flags -----------------------------------------------------
Const OBJECT_INHERIT_ACE = 1
Const CONTAINER_INHERIT_ACE = 2
Const NO_PROPAGATE_INHERIT_ACE = 4
Const INHERIT_ONLY_ACE = 8
Const INHERITED_ACE = 16
Const ACEFLAG_VALID_INHERIT_FLAGS = 31
'Two special flags that pertain only to ACEs that are contained in a SACL are listed below.
Const SUCCESSFUL_ACCESS_ACE_FLAG = 64
Const FAILED_ACCESS_ACE_FLAG = 128

'---- ACE Types -----------------------------------------------------
Const ACCESS_ALLOWED_ACE_TYPE = 0
Const ACCESS_DENIED_ACE_TYPE = 1
Const AUDIT_ACE_TYPE = 2

'---- Access Masks --------------------------------------------------

Dim Perms_LStr, Perms_SStr, Perms_Const
    'Permission LongNames
    Perms_LStr=Array("Synchronize"            , _
            "Take Ownership"                    , _
        "Change Permissions"                , _
        "Read Permissions"                    , _
        "Delete"                            , _
        "Write Attributes"                    , _
        "Read Attributes"                    , _
        "Delete Subfolders and Files"            , _
        "Traverse Folder / Execute File"        , _
        "Write Extended Attributes"            , _
        "Read Extended Attributes"            , _
        "Create Folders / Append Data"        , _
        "Create Files / Write Data"            , _
        "List Folder / Read Data"    )
    'Permission Single Character codes
    Perms_SStr=Array("E"        , _
        "D"        , _
        "C"        , _
        "B"        , _
        "A"        , _
        "9"        , _
        "8"        , _
        "7"        , _
        "6"        , _
        "5"        , _
        "4"        , _
        "3"        , _
        "2"        , _
        "1"        )
    'Permission Integer
    Perms_Const=Array(&H100000    , _
        &H80000        , _
        &H40000        , _
        &H20000        , _
        &H10000        , _
        &H100        , _
        &H80        , _
        &H40        , _
        &H20        , _
        &H10        , _
        &H8            , _
        &H4            , _
        &H2            , _
        &H1        )

   Dim OverLook_Perms_Const
    'Permission Integer
    OverLook_Perms_Const=Array(&H80000000            , _
            &H40000000                                    , _
            &H20000000                                    , _
            &H10000000                                    )

'---- Main Script ---------------------------------------------------

Err.Clear

'Prompt for elevation if UAC is enabled and if no arguments have been passed thourgh the command-line
If WScript.Arguments.Count <> 0 Then strFirstArg = WScript.Arguments(0)
If (Not UACEnabled = 1) OR ((WScript.Arguments.Count <> 0) AND (ScriptEngine = "cscript.exe")) OR ((ScriptEngine = "wscript.exe") AND (Trim(strFirstArg) = "GTsb6vgR1gayegy876FadqXXF7STUghs")) Then
	
	If (ScriptEngine = "wscript.exe") Or ((ScriptEngine = "cscript.exe") AND (WScript.Arguments.Count = 0)) Then
		boolCommandLine = False
		Call SetRootFolder	'prompts the user to set the root folder
		Call SetPriority	'prompts the user to set the task priority level
		Call SetVerbosity	'prompts the user to set the verbosity level
		Call SetRecursion	'prompts the user to set the recursion level
	Else
		boolCommandLine = True
		Call CommandLineParser
	End If

	If boolCommandLine = True Then
		wscript.echo Now & " - Operation Started"
		If Not boolV = True Then
			wscript.echo "Please wait..."
		End If
	End If

	If FolderByUserIsRootDrive(FolderByUser) Then 'needed to scan a whole drive starting at the root
		RootDriveScan = True
		PreviousSlashes = Len(FolderbyUser) - Len(Replace(Folderbyuser, "\", ""))
	Else
		RootDriveScan = False
		PreviousSlashes = Len(FolderbyUser) - Len(Replace(Folderbyuser, "\", "")) +1
	End If

	RootSlashes = Len(FolderbyUser) - Len(Replace(Folderbyuser, "\", ""))	'with this we can easily control folder recursion

	Set ObjLog = ObjFSO.CreateTextFile(logfile, ForWriting)	'Create or overwrite the output log

	Call HTMLOpening	'Open HTML File

	If Verbositylevel = 0 Then
		Call PrintRoot 'Print Root Details in the body section if verbosity is set.
	End If

	GoSubFolders FSO.GetFolder(FolderByUser)	'Loop through all subfolders

	Call CheckOutput 'if verbosity is set to 0 check for blank output

	Call HTMLEnding	'Close HTML File

	If boolCommandLine Then
		Wscript.Echo nfolders & " folder(s) processed" & vbCrLF & "Output file: " & logfile & vbCrLF & Now & " - Operation Succesfully Completed" & vbcrlf
	Else
		Answer = MsgBox (Now & vbCrLF & "Operation Succesfully Completed." & vbcrlf & nfolders & " folder(s) processed." & VbCrLf & "Do you want to open the output now?" & VbCrLf, vbInformation + vbYesNo, "Operation Completed")
		If Answer = vbYes Then
			ObjShell.Run "iexplore.exe " & logfile
		End If
	End If

	Set ObjShell = Nothing		
Else
	'Restart the script passing a bogus argument with a leading blank space
	objShellApp.ShellExecute "wscript.exe", Chr(34) & WScript.ScriptFullName & Chr(34) & " GTsb6vgR1gayegy876FadqXXF7STUghs", "", "runas", 1
End If

Wscript.Quit(0)	'Quit

'---- Open HTML File ------------------------------------------------
Sub HTMLOpening

	objLog.writeline "<!DOCTYPE html PUBLIC " & chr(34) & "-//W3C//DTD XHTML 1.0 Strict//EN" & chr(34) & " " & chr(34) & "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" & chr(34) & ">"
	objLog.writeline "<html>"
	objLog.writeline "<head>"
	objLog.writeline "<!-- saved from url=(0014)about:internet -->"
	objLog.writeline "<title>" & ScriptName & "</title>"
	objLog.writeline "<script type=" & chr(34) & "text/javascript" & chr(34) & " language=" & chr(34) & "JavaScript" & chr(34) & "><!--"
	objLog.writeline "function InsertContent(tid) {"
	objLog.writeline "if(document.getElementById(tid).style.display == " & chr(34) & "none" & chr(34) & ") {"
	objLog.writeline "	document.getElementById(tid).style.display = " & chr(34) & "" & chr(34) & ";"
	objLog.writeline "	}"
	objLog.writeline "else {"
	objLog.writeline "	document.getElementById(tid).style.display = " & chr(34) & "none" & chr(34) & ";"
	objLog.writeline "	}"
	objLog.writeline "}"
	objLog.writeline "function expandAll(txt) {"
	objLog.writeline "	var items = document.getElementsByTagName('div');"
	objLog.writeline "	for(var i=0;i<items.length;i++)"
	objLog.writeline "	{"
	objLog.writeline "		var obj=items[i];"
	objLog.writeline "		if(obj.id.indexOf('details')!=-1)"
	objLog.writeline "		{"
	objLog.writeline "				obj.style.display = " & chr(34) & "" & chr(34) & ";"
	objLog.writeline "		}"
	objLog.writeline "	}"
	objLog.writeline "}"
	objLog.writeline "function collapseAll(txt) {"
	objLog.writeline "	var items = document.getElementsByTagName('div');"
	objLog.writeline "	for(var i=0;i<items.length;i++)"
	objLog.writeline "	{"
	objLog.writeline "		var obj=items[i];"
	objLog.writeline "		if(obj.id.indexOf('details')!=-1)"
	objLog.writeline "		{"
	objLog.writeline "				obj.style.display = " & chr(34) & "none" & chr(34) & ";"
	objLog.writeline "		}"
	objLog.writeline "	}"
	objLog.writeline "}"
	objLog.writeline "//--></script>"
	objLog.writeline "<style type=" & chr(34) & "text/css" & chr(34) & ">"
	objLog.writeline "html{"
	objLog.writeline "	margin:0;"
	objLog.writeline "	padding:0;"
	objLog.writeline "	height:100%;"
	objLog.writeline "	overflow:hidden;" 
	objLog.writeline "	_padding:140px 0px 0px 0px;"
	objLog.writeline "}"
	objLog.writeline "body{"
	objLog.writeline "	margin:0;"
	objLog.writeline "	padding:0;"
	objLog.writeline "	height:100%;"
	objLog.writeline "	overflow:hidden;"
	objLog.writeline "	zoom:1;"
	objLog.writeline "}"
	objLog.writeline "a, a:link, a:visited, a:hover {"
	objLog.writeline "	color:#36393D;"
	objLog.writeline "}"
	objLog.writeline ".frame{"
	objLog.writeline "	position:absolute;"
	objLog.writeline "	top:0;"
	objLog.writeline "	left:0;"
	objLog.writeline "	right:0;"
	objLog.writeline "	bottom:0;"
	objLog.writeline "	overflow:hidden;"
	objLog.writeline "	zoom:1;"
	objLog.writeline "	_right:0;"
	objLog.writeline "	_bottom:0;"
	objLog.writeline "	_width:100%;"
	objLog.writeline "	_height:100%;"
	objLog.writeline "	_position:relative;"
	objLog.writeline "}"
	objLog.writeline ".frame-bd{"
	objLog.writeline "  zoom:1;"
	objLog.writeline "}"
	objLog.writeline "#main-header{"
	objLog.writeline "	overflow:hidden;"
	objLog.writeline "	position:absolute;"
	objLog.writeline "	width:100%;"
	objLog.writeline "	height:140px;"
	objLog.writeline "	top:0;"
	objLog.writeline "	left:0;"
	objLog.writeline "	_zoom:1;"
	objLog.writeline "	_overflow:visible;"
	objLog.writeline "	_padding:0 0px 0 0;"
	objLog.writeline "}"
	objLog.writeline "#main-header .frame{"
	objLog.writeline "	overflow:auto;"
	objLog.writeline "	_padding:0 0 0 0px;"
	objLog.writeline "	_zoom:1;"
	objLog.writeline "}"
	objLog.writeline "#main-header .frame .frame-bd{"
	objLog.writeline "	zoom:1;"
	objLog.writeline "	_margin:0 0 0 -0px;"
	objLog.writeline "}"
	objLog.writeline "#pri-main{"
	objLog.writeline "	overflow:hidden;"
	objLog.writeline "	position:absolute;"
	objLog.writeline "	top:140px;"
	objLog.writeline "	left:0px;"
	objLog.writeline "	right:0px;"
	objLog.writeline "	bottom:0px;"
	objLog.writeline "	z-index:2;"
	objLog.writeline "	_top:140px;"
	objLog.writeline "	_bottom:auto;"
	objLog.writeline "	_height:100%;"
	objLog.writeline "}"
	objLog.writeline "#pri-main .frame{"
	objLog.writeline "	overflow:auto;"
	objLog.writeline "}"
	objLog.writeline "body{"
	objLog.writeline "	font-family:Arial, " & chr(34) & "Lucida Grande" & chr(34) & ", sans-serif;"
	objLog.writeline "	line-height:1.3em;"
	objLog.writeline "	font-size:0.9em;"
	objLog.writeline "	letter-spacing:0.01em;"
	objLog.writeline "	color:#36393D;"
	objLog.writeline "	font-weight:normal;"
	objLog.writeline "}"
	objLog.writeline ".frame{"
	objLog.writeline "	background:#fff 0 0 no-repeat;"
	objLog.writeline "	_background-attachment:fixed;"
	objLog.writeline "	border:#F7FAFF 1px solid;"
	objLog.writeline "}"
	objLog.writeline "li {"
	objLog.writeline "	list-style-type:none;"
	objLog.writeline "}"
	objLog.writeline ".details_inh{"
	objLog.writeline "	font-size: 0.8em;"
	objLog.writeline "	line-height:1.0em;"
	objLog.writeline "	position:relative;"
	objLog.writeline "	float:left;"
	objLog.writeline "	border:2px dashed #008C00;"
	objLog.writeline "	padding:10px;"
	objLog.writeline "	margin:10px;"
	objLog.writeline "}"
	objLog.writeline ".details_noinh{"
	objLog.writeline "	font-size: 0.8em;"
	objLog.writeline "	line-height:1.0em;"
	objLog.writeline "	position:relative;"
	objLog.writeline "	float:left;"
	objLog.writeline "	border:2px dashed #CC0000;"
	objLog.writeline "	padding:10px;"
	objLog.writeline "	margin:10px;"
	objLog.writeline "}"
	objLog.writeline ".details_mod{"
	objLog.writeline "	font-size: 0.8em;"
	objLog.writeline "	line-height:1.0em;"
	objLog.writeline "	position:relative;"
	objLog.writeline "	float:left;"
	objLog.writeline "	border:2px dashed #4096EE;"
	objLog.writeline "	padding:10px;"
	objLog.writeline "	margin:10px;"
	objLog.writeline "}"
	objLog.writeline ".details_notrecinh{"
	objLog.writeline "	font-size: 0.8em;"
	objLog.writeline "	line-height:1.0em;"
	objLog.writeline "	position:relative;"
	objLog.writeline "	float:left;"
	objLog.writeline "	border:2px dashed #36393D;"
	objLog.writeline "	padding:10px;"
	objLog.writeline "	margin:10px;"
	objLog.writeline "}"
	objLog.writeline ".noinheritance {"
	objLog.writeline "	color:#CC0000;"
	objLog.writeline "	font-weight:bold;"
	objLog.writeline "}"
	objLog.writeline ".inheritance {"
	objLog.writeline "	color:#008C00;"
	objLog.writeline "	font-weight:bold;"
	objLog.writeline "}"
	objLog.writeline ".modinheritance {"
	objLog.writeline "	color:#4096EE;"
	objLog.writeline "	font-weight:bold;"
	objLog.writeline "}"
	objLog.writeline ".notrecinheritance {"
	objLog.writeline "	color:#36393D;"
	objLog.writeline "	font-weight:bold;"
	objLog.writeline "}"
	objLog.writeline ".bold {"
	objLog.writeline "	font-weight:bold;"
	objLog.writeline "}"
	objLog.writeline ".small {"
	objLog.writeline "	font-size: 0.8em;"
	objLog.writeline "}"
	objLog.writeline "p {"
	objLog.writeline "	margin-left:0.5em;"
	objLog.writeline "	margin-top:0.2em;"
	objLog.writeline "	margin-bottom:0.4em;"
	objLog.writeline "}"
	objLog.writeline ".denied {"
	objLog.writeline "	color:#CC0000;"
	objLog.writeline "}"
	objLog.writeline ".allowed {"
	objLog.writeline "	color:#008C00;"
	objLog.writeline "}"
	objLog.writeline "</style>"
	objLog.writeline "</head>"
	objLog.writeline "<body>"
	objLog.writeline "<div id=" & chr(34) & "doc" & chr(34) & ">"
	objLog.writeline "	<!--main-header-->"
	objLog.writeline "	<div id=" & chr(34) & "main-header" & chr(34) & ">"
	objLog.writeline "		<div class=" & chr(34) & "frame" & chr(34) & ">"
	objLog.writeline "			<div class=" & chr(34) & "frame-bd" & chr(34) & ">"
	objLog.writeline "				<p><span class=" & chr(34) & "bold" & chr(34) & ">"& ScriptName & "</span><br />"
	objLog.writeline "				Report generated on: "& FormatDateTime(Date(),1) & ", Time: " & Time & "</p>"
	objLog.writeline "				<p><span class=" & chr(34) & "small" & chr(34) & "><span class=" & chr(34) & "bold" & chr(34) & ">Inheritance:</span> &nbsp;</span> <span class =" & chr(34) & "noinheritance" & chr(34) & ">&#215;</span> <span class=" & chr(34) & "small" & chr(34) & ">Disabled &nbsp;</span> <span class =" & chr(34) & "modinheritance" & chr(34) & ">&#43;</span> <span class=" & chr(34) & "small" & chr(34) & ">Enabled with modified DACL &nbsp;</span> <span class =" & chr(34) & "inheritance" & chr(34) & ">&#61;</span> <span class=" & chr(34) & "small" & chr(34) & ">Enabled &nbsp;</span></p>"'<span class =" & chr(34) & "notrecinheritance" & chr(34) & ">&#33;</span> <span class=" & chr(34) & "small" & chr(34) & ">Not Recognized</span></p>"
	objLog.writeline "			</div>"
	objLog.writeline "			<p align=right><a href=" & chr(34) & "#" & chr(34) & " onclick=" & chr(34) & "expandAll(this)" & chr(34) & ">Expand All</a>&nbsp;\&nbsp;<a href=" & chr(34) & "#" & chr(34) & " onclick=" & chr(34) & "collapseAll(this)" & chr(34) & ">Collapse All</a></p>"
	
	If Verbositylevel = 1 Then 
		ObjLog.writeline "			<p>Root Folder: "& FolderByUser & "</p>"
	Else If Recursionlevel <> 0 Then 
		ObjLog.writeline "			<p>The script found the following interesting folder(s) limiting the search process to " & RecursionLevel& " sublevel(s):</p>"
		Else 
			ObjLog.writeline "		<p>The script found the following interesting folder(s):</p>"
		End If
	End If
	
	ObjLog.writeline "		</div>"
	objLog.writeline "	</div>"
	objLog.writeline "<!--//main-header-->"
	objLog.writeline "<!--pri-->"
	objLog.writeline "<div id=" & chr(34) & "pri" & chr(34) & ">"
	objLog.writeline "		<!--pri-main-->"
	objLog.writeline "		<div id=" & chr(34) & "pri-main" & chr(34) & ">"
	objLog.writeline "			<div class=" & chr(34) & "frame" & chr(34) & ">"
	objLog.writeline "				<div class=" & chr(34) & "frame-bd" & chr(34) & ">"
	objLog.writeline "					<div class=" & chr(34) & "bd" & chr(34) & ">"
	objLog.writeline "					<ul>"

End Sub

'---- Recurse Subfolders --------------------------------------------

Sub GoSubFolders (Folder)

	Wscript.Sleep(SleepTime)	' related to task priority level
	
	If (boolV = True) AND (boolCommandLine = True) Then
		wscript.echo Folder
	End If

	For Each Subfolder in Folder.SubFolders
	On Error Resume Next
		RecursionSlashes = Len(Folder) - Len(Replace(Folder, "\", ""))
		If (RootDriveScan = True) And (Len(Folder) - Len(FolderByUser) <> 0) Then 'needed to scan a whole drive starting at the root
			NewRootSlashes = 0
		Else
			NewRootSlashes = RootSlashes	
		End If
		If ((trim(RecursionSlashes) - trim(NewRootSlashes)) >= RecursionLevel) And (RecursionLevel <> 0) Then
			Exit For
		Else
			nfolders = nfolders + 1
			Call GetParentACL(Folder) 
			Call GetChildACL(subfolder)
			If Verbositylevel = 1 then	'If Verbositylevel has been set to 1 print details of all folders
				tempInheritance = Inheritance(SubFolder)
				divID = divID +1
				NewSlashes = Len(Subfolder) - Len(Replace(Subfolder, "\", ""))
				Slashes = trim(PreviousSlashes - NewSlashes)
				PreviousSlashes = trim(NewSlashes)  
				If Slashes < 0 Then
					ObjLog.Writeline "<ul>"
					Else If Slashes > 0 Then
						Do Until Slashes  = 0
							Slashes = Slashes - 1
							ObjLog.Writeline "</ul>"
						Loop
					End If
				End If

				ObjLog.Writeline "<li><div style=" & chr(34) & "float:left; width:90%;" & chr(34) & ">" & tempInheritance & "&nbsp;<a href=" & chr(34) & "javascript:InsertContent('details" & divID & "');" & chr(34) & ">"
				ObjLog.Writeline(strReverse(Left(strReverse(SubFolder), Instr(strReverse(SubFolder) , "\"))) & "</a></div>")	'print folder path
				ObjLog.Writeline "<div id=" & chr(34) & "details" & divID & chr(34) & " style=" & chr(34) & "display: none;" & chr(34) & " class=" & chr(34) & details_style & chr(34) & ">"
				ObjLog.Writeline("<p><span class=" & chr(34) & "bold" & chr(34) & ">" & SubFolder & "</span></p>")
				ObjLog.Writeline("<p>")
				ObjLog.Writeline("<em>Creation Date:&nbsp;</em>" & SubFolder.DateCreated & "<br />")
				ObjLog.Writeline("<em>Last Modified:&nbsp;</em>" & SubFolder.DateLastModified)
				ObjLog.Writeline("</p>")  

				ObjLog.Writeline("<p><em>Owner:&nbsp;</em>" & GetOwner(subfolder) & "</p>")  

				If printParent = True Then
					Call PrintParentDetails	'print parent ace array
				End if

				Call PrintChildDetails	'print child ace array -current folder-

				ObjLog.Writeline "</div><div style=" & chr(34) & "clear:both;" & chr(34) & " />"
				details_style = ""
				ObjLog.Writeline "</li>"
			Else	'If Verbositylevel has been set to 0 print only modified DACLs
				tempInheritance = Inheritance(SubFolder)	
				If printParent = True Then
					divID = divID +1
					ObjLog.Writeline "<li><div style=" & chr(34) & "float:left; width:90%;" & chr(34) & ">" & tempInheritance & "&nbsp;<a href=" & chr(34) & "javascript:InsertContent('details" & divID & "');" & chr(34) & ">"
					ObjLog.Writeline(strReverse(Left(strReverse(SubFolder), Instr(strReverse(SubFolder) , "\"))) & "</a></div>")	'print folder path
					ObjLog.Writeline "<div id=" & chr(34) & "details" & divID & chr(34) & " style=" & chr(34) & "display: none;" & chr(34) & " class=" & chr(34) & details_style & chr(34) & ">"
					ObjLog.Writeline("<p><span class=" & chr(34) & "bold" & chr(34) & ">" & SubFolder & "</span></p>")
					ObjLog.Writeline("<p>")
					ObjLog.Writeline("<em>Creation Date:&nbsp;</em>" & SubFolder.DateCreated & "<br />")
					ObjLog.Writeline("<em>Last Modified:&nbsp;</em>" & SubFolder.DateLastModified)
					ObjLog.Writeline("</p>")  

					ObjLog.Writeline("<p><em>Owner:&nbsp;</em>" & GetOwner(subfolder) & "</p>")  
			
					Call PrintParentDetails	'print parent ace array
					Call PrintChildDetails	'print child ace array -current folder-

					ObjLog.Writeline "</div><div style=" & chr(34) & "clear:both;" & chr(34) & " />"
					details_style = ""
					ObjLog.Writeline "</li>"
				End if		
			End If

			GoSubFolders Subfolder	'Recall itself recursively.

		End If

	Next

End Sub

'---- GetChildACL Sub -----------------------------------------------

Sub GetChildACL(strPath)

	Dim objFileSecSetting, objOutParams, objSecDescriptor, objOwner, objDACL_Member
	Dim objtrustee, numAceFlags, strAceFlags, x, strAceType, numControlFlags, ReturnAceFlags, TempSECString
	ReDim arrayChildACLs(0)

	Do
		On Error Resume Next
		Set objFileSecSetting = objService.Get("Win32_LogicalFileSecuritySetting.Path=""" & Replace(strPath,"\","\\") & """")
'		If Err.Number <> 0 Then
'			If BoolCommandLine Then
'				Wscript.Echo "Error: " & Err.Number & " - " & Err.Description & " in " & strPath
'				Wscript.Quit(1)	'Quit
'			Else
'				MsgBox "Error: " & Err.Number & " - " & Err.Description & " in " & strPath, vbCritical + vbOKOnly, "Error"
'				Wscript.Quit(1)	'Quit
'			End If
'		End If
		Err.Clear
		On Error GoTo 0
		Set objOutParams = objFileSecSetting.ExecMethod_("GetSecurityDescriptor")
		Set objSecDescriptor = objOutParams.Descriptor
		numControlFlags = objSecDescriptor.ControlFlags
		If IsArray(objSecDescriptor.DACL) Then
			For Each objDACL_Member In objSecDescriptor.DACL
				TempSECString = ""
				ReturnAceFlags = 0
				Select Case objDACL_Member.AceType
					Case ACCESS_ALLOWED_ACE_TYPE
						strAceType = "<span class=" & Chr(34)& "allowed" & Chr(34)&">Allowed</span>&nbsp;"
					Case ACCESS_DENIED_ACE_TYPE
						strAceType = "<span class=" & Chr(34)& "denied" & Chr(34)&">Denied</span>&nbsp;&nbsp;"
					Case Else
						strAceType = "Unknown&nbsp;&nbsp;"
				End Select
				Set objtrustee = objDACL_Member.Trustee
				numAceFlags = objDACL_Member.AceFlags
				strAceFlags = StringAceFlag(numAceFlags, numControlFlags, SE_DACL_AUTO_INHERITED, FALSE, ReturnAceFlags)
				TempSECString = SECString(objDACL_Member.AccessMask,TRUE)
				If ReturnAceFlags = 2 Then
					If TempSECString = "Read and Execute" Then
						TempSECString = "List Folder Contents"
					End If
				End If
				Call AddStringToArray(arrayChildACLs, strPackString(strAceType, 9, 1, FALSE) & strPackString(objtrustee.Domain & "\" & objtrustee.Name & "&nbsp;", 24, 1, FALSE) & strPackString(TempSECString, 22, 1, FALSE) & strPackString(strAceFlags, 22, 1, FALSE),-1)
				Set objtrustee = Nothing
			Next
		Else
			'ObjLog.Writeline("<p>No Permissions set</p>") ' For Debugging Purposes only
		End If

		Exit Do	'We really didn't want to Loop
	Loop

	'Clear Objects
	Set objSecDescriptor = Nothing
	Set objDACL_Member = Nothing
	Set objtrustee = Nothing
	Set objOutParams = Nothing
	Set objFileSecSetting = Nothing

End Sub

'---- GetParentACL Sub ----------------------------------------------

Sub GetParentACL(strPath)

	Dim objFileSecSetting, objOutParams, objSecDescriptor, objOwner, objDACL_Member
	Dim objtrustee, numAceFlags, strAceFlags, x, strAceType, numControlFlags, ReturnAceFlags, TempSECString
	ReDim arrayParentACLs(0)

	Do
		On Error Resume Next
		Set objFileSecSetting = objService.Get("Win32_LogicalFileSecuritySetting.Path=""" & Replace(strPath,"\","\\") & """")
'		If Err.Number <> 0 Then
'			If boolCommandLine Then
'				Wscript.Echo "Error: " & Err.Number & " - " & Err.Description & " in " & strPath
'				Wscript.Quit(1)	'Quit
'			Else
'				MsgBox "Error: " & Err.Number & " - " & Err.Description & " in " & strPath, vbCritical + vbOKOnly, "Error"
'				Wscript.Quit(1)	'Quit
'			End If
'		End If
		Err.Clear
		On Error GoTo 0	
		Set objOutParams = objFileSecSetting.ExecMethod_("GetSecurityDescriptor")
			Set objSecDescriptor = objOutParams.Descriptor
		numControlFlags = objSecDescriptor.ControlFlags
		If IsArray(objSecDescriptor.DACL) Then   
			For Each objDACL_Member in objSecDescriptor.DACL
				TempSECString = ""
				ReturnAceFlags = 0
				Select Case objDACL_Member.AceType
					Case ACCESS_ALLOWED_ACE_TYPE
						strAceType = "<span class=" & Chr(34)& "allowed" & Chr(34)&">Allowed</span>&nbsp;"
					Case ACCESS_DENIED_ACE_TYPE
						strAceType = "<span class=" & Chr(34)& "denied" & Chr(34)&">Denied</span>&nbsp;&nbsp;"
					Case Else
						strAceType = "Unknown&nbsp;&nbsp;"
				End Select
				Set objtrustee = objDACL_Member.Trustee
				numAceFlags = objDACL_Member.AceFlags
				strAceFlags = StringAceFlag(numAceFlags, numControlFlags, SE_DACL_AUTO_INHERITED, FALSE, ReturnAceFlags)
				TempSECString = SECString(objDACL_Member.AccessMask,TRUE)
				If ReturnAceFlags = 2 Then
					If TempSECString = "Read and Execute" Then
						TempSECString = "List Folder Contents"
					End If
				End If
				Call AddStringToArray(arrayParentACLs, strPackString(strAceType, 9, 1, FALSE) & strPackString(objtrustee.Domain & "\" & objtrustee.Name & "&nbsp;", 24, 1, FALSE) & strPackString(TempSECString, 22, 1, FALSE) & strPackString(strAceFlags, 22, 1, FALSE),-1)
				Set objtrustee = Nothing
			Next
		Else
			'ObjLog.Writeline("<p>No Permissions set</p>") ' For Debugging Purposes only
		End If

		Exit Do	'We really didn't want to loop
	Loop

	'Clear Objects
	Set objSecDescriptor = Nothing
	Set objDACL_Member = Nothing
	Set objtrustee = Nothing
	Set objOutParams = Nothing
	Set objFileSecSetting = Nothing

End Sub

'---- SECString Function --------------------------------------------

Function SECString(byval intBitmask, byval ReturnLong)

	Dim LongName, X
	SECString = ""
	
	Do
		For X = LBound(Perms_LStr) to UBound(Perms_LStr)
			If ((intBitmask And Perms_Const(X)) = Perms_Const(X)) Then
				If Perms_SStr(X) <> "" Then
					SECString = SECString & Perms_SStr(X)
				End If
			End If
		Next

		Select Case SECString
			Case "DCBA987654321", "EDCBA987654321"
				SECString = "F"                                'Full control
				LongName = "Full Control"   
			Case "BA98654321", "EBA98654321"
				SECString = "M"                                'Modify
				LongName = "Modify"
			Case "B98654321", "EB98654321"
				SECString = "XW"                                'Read, Write and Execute
				LongName = "Read, Write and Execute"
			Case "B9854321", "EB9854321"
				SECString = "RW"                                'Read and Write
				LongName = "Read and Write"
			Case "B8641", "EB8641"
				SECString = "X"                                'Read and Execute
				LongName = "Read and Execute"
			Case "B841", "EB841"
				SECString = "R"                                'Read
				LongName = "Read"
			Case "9532", "E9532"
				SECString = "W"                                'Write
				LongName = "Write"
			Case Else
				If SECString = "" Then
					LongName = "Special (Unknown)"  
				Else
					If LEN(SECString) = 1 Then
						For X = LBound(Perms_SStr) to UBound(Perms_SStr)
							If StrComp(SECString,Perms_SStr(X),1) = 0 Then
								LongName = "Advanced (" & Perms_LStr(X) & ")"
								Exit For
							End If
						Next
					Else
						LongName = "Special (" & SECString & ")"
					End If
				End If
		End Select

		Exit Do
	Loop

	If ReturnLong Then SECString = LongName

End Function

'---- StringAceFlag Function ----------------------------------------

Function StringAceFlag(ByVal numAceFlags, ByVal numControlFlags, ByVal FlagToCheck, ByVal ReturnShort, ByRef ReturnAceFlags)

	Dim TempShort, TempLong
	Do
		If numAceFlags = 0 Then
			TempShort = "Implicit"
			TempLong = "This Folder Only"
			Exit Do
		End If
		If numAceFlags > FAILED_ACCESS_ACE_FLAG Then
			numAceFlags = numAceFlags - FAILED_ACCESS_ACE_FLAG
		End If
		If numAceFlags > SUCCESSFUL_ACCESS_ACE_FLAG Then
			numAceFlags = numAceFlags - SUCCESSFUL_ACCESS_ACE_FLAG
		End If
		If ((numAceFlags And INHERITED_ACE) = INHERITED_ACE) Then
			TempShort = "Inherited"
			numAceFlags = numAceFlags - INHERITED_ACE
			TempLong = "Inherited"
		Else
			TempShort = "Implicit"
			TempLong = "Implicit"
		End If

		ReturnAceFlags = numAceFlags

		If numControlFlags > DENY_INHERIT Then
			numControlFlags = numControlFlags - DENY_INHERIT
		End If
		If numControlFlags > ALLOW_INHERIT Then
			numControlFlags = numControlFlags - ALLOW_INHERIT
		End If

		Select Case numAceFlags
			Case 0
				TempLong = "This Folder Only"
			Case 1                            'OBJECT_INHERIT_ACE
				TempLong = "This Folder and Files"
			Case 2                            'CONTAINER_INHERIT_ACE
				TempLong = "This Folder and Subfolders"
			Case 3
				TempLong = "This Folder, Subfolders and Files"
			Case 9
				TempLong = "Files Only"
			Case 10
				TempLong = "Subfolders only"
			Case 11
				TempLong = "Subfolders and Files only"
			Case Else
				If ((numControlFlags And FlagToCheck) = FlagToCheck) Then
					TempShort = "Inherited"
					TempLong = "Inherited"
				End If
		End Select

		Exit Do 
	Loop

	If ReturnShort Then
		StringAceFlag = TempShort & "<br />"
	Else
		StringAceFlag = TempLong & "<br />"
	End If

End Function

'---- AddStringToArray Function -------------------------------------

Function AddStringToArray(ByRef theArray, byval theMember, ByVal intUseIndex)

	Dim UseThisNumber
	Do
		AddStringToArray = UBound(theArray)
		If intUseIndex <> -1 Then
			If intUseIndex > AddStringToArray Then
				AddStringToArray = intUseIndex
			End If
			UseThisNumber = intUseIndex
		Else
			'We will always increment by 1 so the first member is 0 or blank
			AddStringToArray = AddStringToArray + 1
			UseThisNumber = AddStringToArray
		End If

		ReDim Preserve theArray(AddStringToArray)
		theArray(UseThisNumber) = theMember

		Exit Do
	Loop

End Function

'---- strPackString Function ----------------------------------------

Function strPackString(byval strString, ByVal intWidth, byval blnAfter, byval blnTruncate)

	Do
		intWidth = CInt(intWidth)
		blnAfter = CBool(blnAfter)
		blnTruncate = CBool(blnTruncate)

		If IsNull(strString) Then
			strPackString = "null" & Space(intWidth-4)
			Exit Do
		End If

		strString = CStr(strString)

		If intWidth > Len(strString) Then
			If blnAfter Then
				strPackString = strString & Space(intWidth-Len(strString))
			Else
				strPackString = Space(intWidth-Len(strString)) & strString & " "
			End If
		Else
			If blnTruncate Then
				strPackString = Left(strString, intWidth-1) & " "
			Else
				strPackString = strString & " "
			End If
		End If

		Exit Do
	Loop

End Function

'---- Inheritance Function ------------------------------------------

Function Inheritance(Folder)

	On Error Resume Next
	Set objFileSecSetting = objService.Get("Win32_LogicalFileSecuritySetting.Path=""" & Replace(Folder,"\","\\") & """")
	If Err.Number <> 0 Then
'		If boolCommandLine Then
'			Wscript.Echo "Error: " & Err.Number & " - " & Err.Description & " in " & Folder
'			Wscript.Quit(1)	'Quit
'		Else
'			MsgBox "Error: " & Err.Number & " - " & Err.Description & " in " & Folder, vbCritical + vbOKOnly, "Error"
'			Wscript.Quit(1)	'Quit
'		End If
		'On error apply noinheritance style and exit the function
		Inheritance = "<span class =" & Chr(34) & "noinheritance" & Chr(34) &">&#215;</span>"
		details_style = "details_noinh"
		printParent = True
		Exit Function
	End If
	Err.Clear
	On Error GoTo 0

	printParent = False
	objFileSecSetting.GetSecurityDescriptor objSecurityDescriptor
	If objSecurityDescriptor.ControlFlags AND  SE_DACL_PROTECTED Then
		Inheritance = "<span class =" & Chr(34) & "noinheritance" & Chr(34) &">&#215;</span>"
		details_style = "details_noinh"
		printParent = True
	Else
		differences = 0
		For Each strElementFirst In ArrayChildACLS
			blnExistinSecond = False
			For Each strElementSecond In arrayParentACLs
				If strElementFirst = strElementSecond Then
					blnExistInSecond = True
					Exit For
				End If
			Next
			If Not blnExistInSecond Then
				differences = differences + 1
			End If
		Next		
		For Each strElementFirst In arrayParentACLs
			blnExistinSecond = False
			For Each strElementSecond in ArrayChildACLS
				If strElementFirst = strElementSecond Then
					blnExistInSecond = True
					Exit For
				End If
			Next
			If Not blnExistInSecond Then
				differences = differences + 1
			End If
		Next
		If differences > 0 Then
			Inheritance = "<span class =" & chr(34) & "modinheritance" & chr(34) &">&#43;</span>"
			details_style = "details_mod"
			printParent = True
		Else
			Inheritance = "<span class =" & chr(34) & "inheritance" & chr(34) &  ">&#61;</span>"
			details_style = "details_inh"
		End If
		
	End If

End Function

'---- PrintChildDetails Sub -----------------------------------------

Sub PrintChildDetails

	ObjLog.Writeline "<p><em>Permissions:</em><br />"
	
	'If there was an error arrayChildACLS must be empty, log the error and exit
	If LBound(arrayChildACLS) = UBound(arrayChildACLS) Then
		ObjLog.WriteLine "<em>*unable to get security information*</em>"
	Else
		For x = LBound(arrayChildACLS) To UBound(arrayChildACLS)
			ObjLog.Writeline(arrayChildACLS(x))
		Next
	End If

	ObjLog.Writeline("</p>")

End Sub

'---- PrintParentDetails Sub ----------------------------------------

Sub PrintParentDetails

	ObjLog.Writeline "<p><em>Parent Folder Permissions:</em><br />"
	
	'If there was an error arrayParentACLS must be empty, log the error and exit
	If LBound(arrayParentACLS) = UBound(arrayParentACLS) Then
		ObjLog.WriteLine "<em>*unable to get secrity information*</em>"
	Else
		For y = LBound(arrayParentACLS) to UBound(arrayParentACLS)
			ObjLog.Writeline(arrayParentACLS(y))
		Next
	End If
	
	ObjLog.Writeline("</p>")

End Sub

'---- PrintRoot Sub -------------------------------------------------

Sub PrintRoot

	Set objFolder = objFSO.GetFolder(FolderByUser)
	Call GetChildACL(FolderByUser)
	divID = divID +1
	ObjLog.Writeline "<div style=" & chr(34) & "float:left; width:90%;" & chr(34) & "><p>Root Folder:&nbsp;<a href=" & chr(34) & "javascript:InsertContent('details" & divID & "');" & chr(34) & ">"
	ObjLog.Writeline(FolderbyUser & "</a></div>")	'print folder path
	ObjLog.Writeline "<div id=" & chr(34) & "details" & divID & chr(34) & " style=" & chr(34) & "display: none;" & chr(34) & " class=" & chr(34) & "details_notrecinh" & chr(34) & ">"
	ObjLog.Writeline("<p><span class=" & chr(34) & "bold" & chr(34) & ">" & FolderByUser & "</span></p>")
	ObjLog.Writeline("<p>")
	If RootDriveScan = True Then 'needed to scan a whole drive starting at the root
		ObjLog.Writeline("<em>Creation Date:&nbsp;</em>" & "Property not supported for root drives" & "<br />")
		ObjLog.Writeline("<em>Last Modified:&nbsp;</em>" & "Property not supported for root drives")
	Else
		ObjLog.Writeline("<em>Creation Date:&nbsp;</em>" & objFolder.DateCreated & "<br />")
		ObjLog.Writeline("<em>Last Modified:&nbsp;</em>" & objFolder.DateLastModified)
	End If
	ObjLog.Writeline("</p>") 
	ObjLog.Writeline("<p><em>Owner:&nbsp;</em>" & GetOwner(FolderByUser) & "</p>")  
	
	Call PrintChildDetails	'print child ace array -current folder-

	ObjLog.Writeline "</div><div style=" & chr(34) & "clear:both;" & chr(34) & " />"
	details_style = ""
	ObjLog.Writeline "</p>"
				
	Set objFolder = Nothing
	
End Sub

'---- GetOwner Function ---------------------------------------------

Function GetOwner(strpath)

	On Error Resume Next
	Set objFileSecSetting = objService.Get("Win32_LogicalFileSecuritySetting.Path=""" & Replace(strPath,"\","\\") & """")
	If Err.Number <> 0 Then
'		If boolCommandLine Then
'			Wscript.echo "Error: " & Err.Number & " - " & Err.Description & " in " & Folder
'			Wscript.Quit(1)	'Quit
'		Else
'			MsgBox "Error: " & Err.Number & " - " & Err.Description & " in " & Folder, vbCritical + vbOKOnly, "Error"
'			Wscript.Quit(1)	'Quit
'		End If
		'On error close the objects, log the error and exit the function
		Set objFileSecSetting = Nothing
		GetOwner = "<em>*unable to get owner information*</em>"
		Exit Function
	End If
	Err.Clear
	On Error GoTo 0
	Set objOutParams = objFileSecSetting.ExecMethod_("GetSecurityDescriptor")
	Set objSecDescriptor = objOutParams.Descriptor
	Set objOwner = objSecDescriptor.Owner
	
	GetOwner = objOwner.Domain & "\" & objOwner.Name
	
	' Close Objects
	Set objFileSecSetting = Nothing
	Set objOutParams = Nothing
	Set objSecDescriptor = Nothing
	Set objOwner = Nothing

End Function

'---- Sub CheckOutput -----------------------------------------------

Sub CheckOutput

	If (divID <= 1) And (verbosityLevel = 0) Then 
		ObjLog.WriteLine "<p>No interesting folder(s) found.</p>"
	End If
	
End Sub 

'---- Close HTML File -----------------------------------------------

Sub HTMLEnding

	ObjLog.Writeline "					</ul>"
	ObjLog.Writeline "					</div>"
	ObjLog.Writeline "				</div>"
	ObjLog.Writeline "			</div>"
	ObjLog.Writeline "		</div>"
	ObjLog.Writeline "		<!--//pri-main-->"
	ObjLog.Writeline "	</div>"
	ObjLog.Writeline "	<!--//pri-->"
	ObjLog.Writeline "</div>"
	ObjLog.Writeline "</body>"
	ObjLog.Writeline "</html>"

End Sub

'---- Sub SetRootFolder ---------------------------------------------

Sub SetRootFolder

	If FolderByUserIsRootDrive(CurrentPath) then
		FolderByUser = Inputbox("Please enter the root folder's path" & VbCrLf & VbCrLf & "For example C:\Documents and Settings"  &  vbCRLF & vbCRLF & "Press OK to continue or Cancel to quit","Set Root Folder",CurrentPath & "\")
	Else
		FolderByUser = Inputbox("Please enter the root folder's path" & VbCrLf & VbCrLf & "For example C:\Documents and Settings"  &  vbCRLF & vbCRLF & "Press OK to continue or Cancel to quit","Set Root Folder",CurrentPath)
	End If
	If FolderByUser = "" Then	'Cancel button pressed
		WScript.Quit(0)
	Else
		If FolderByUserIsRootDrive(FolderByUser) then
			If Right(FolderByUser,1) <> "\" then
				FolderByUser = FolderByUser & "\"
			End If
		Else
			If Right(FolderByUser,1) = "\" then
				FolderByUser= Left(FolderByUser, Len(FolderByUser)-1)
			End If
		End If
		If ObjFSO.FolderExists(FolderByUser) = False Then
			MsgBox "The specified path does not exist", vbcritical, "Error"
			Call SetRootFolder
		End If
	End If

End Sub

'---- Sub SetPriority -----------------------------------------------

Sub SetPriority

	If Not IsWin2000 Then
		Sleeptime = 0
		Call SetLowPriorityProc
	Else
		Priority = InputBox("*** Windows 2000 only ***" & VbCrLf & "Please set the task priority level: 1, 0 or 2" & VbCrLf & VbCrLf & "1 = normal - standard execution, normal cpu usage" & vbCRLF & "0 = low - slower execution but lower cpu usage" & vbCRLF & "2 = high - faster execution but higher cpu usage" & vbCRLF & vbCRLF & "Press OK to continue or Cancel to quit","Set Task Priority","1")
		If Priority = "" Then	'Cancel button pressed
			WScript.Quit(0)	'Quit
		Else
			Select Case Priority
				Case "0"
					SleepTime = 400
				Case "1"
					Sleeptime = 150
				Case "2"
					Sleeptime = 0
				Case Else
					MsgBox "Invalid option, press OK and retry.", vbcritical, "Error"
					Call SetPriority
			End Select
		End If	
	End If

End Sub

'---- Sub SetVerbosity ----------------------------------------------

Sub SetVerbosity

	Verbosity = InputBox("Please set the verbosity level: 0 or 1" & VbCrLf & VbCrLf & "0 = normal - print only interesting folders" & vbCRLF & "1 = high - print all folders, even not interesting ones" & vbCRLF & vbCRLF & "Press OK to continue or Cancel to quit","Set Verbosity","0")
	If Verbosity = "" Then	'Cancel button pressed
		WScript.Quit(0)	'Quit
	Else
		Select Case Verbosity
			Case "0"
				Verbositylevel = 0
			Case "1"
				Verbositylevel = 1
			Case Else
				MsgBox "Invalid option, press OK and retry.", vbcritical, "Error"
				Call SetVerbosity
		End Select
	End If

End Sub

'---- Sub SetRecursion ----------------------------------------------

Sub SetRecursion

	Recursion = InputBox("Please set the recursion level" & VbCrLf & VbCrLf & "leave 0 to analyze the entire folders tree " & vbCRLF & vbCRLF & "Press OK to continue or Cancel to quit","Set Recursion","0")
	If Recursion = "" Then	'Cancel button pressed
		WScript.Quit(0)	'Quit
	Else
		If Recursion < 0 Then
				MsgBox "Invalid option, press OK and retry.", vbcritical, "Error"
				Call SetRecursion
		End If
		RecursionLevel = CInt(Trim(Recursion))
	End If

End Sub

'---- Sub SetLowPriorityProc ----------------------------------------

Sub SetLowPriorityProc

		Set objWMI = GetObject("winmgmts:\\.\root\cimv2")
		Set ColProcs = objWMI.ExecQuery("SELECT * FROM Win32_Process " & "WHERE Name='" & ScriptEngine & "'")
		For Each objProc in ColProcs
			If (InStr(LCase(objProc.CommandLine), LCase(WScript.ScriptName)) > 0) Then
				objProc.SetPriority 64 'Low Priority
			End If
		Next
		Set objWMI = Nothing

End Sub

'---- IsWin2000 Function --------------------------------------------

Function IsWin2000

	Dim TmpObjWMI, TmpObjOS, TmpColOSs
	IsWin2000 = False
	Set TmpObjWMI = GetObject("winmgmts:\\.\root\cimv2")
	Set TmpColOSs = TmpObjWMI.ExecQuery ("Select * from Win32_OperatingSystem")
	For Each TmpObjOS in TmpColOSs
		If (Instr(LCase(TmpObjOS.Caption), Lcase("2000")) > 0) Then
			IsWin2000 = True
		End If
	Next
	Set TmpObjWMI = Nothing
	
End Function

'---- FolderByUserIsRootDrive Function ------------------------------

Function FolderByUserIsRootDrive(FolderByUser)

	FolderByUserIsRootDrive = False
	Set colDrives = objFSO.Drives
	For Each objDrive in colDrives
		If (trim(lcase(objDrive.DriveLetter)) = trim(lcase(FolderByUser))) OR (trim(lcase(objDrive.DriveLetter)) & ":" = trim(lcase(FolderByUser))) OR (trim(lcase(objDrive.DriveLetter)) & ":\" = trim(lcase(FolderByUser))) Then
			FolderByUserIsRootDrive = True
			Exit for
		Else
			FolderByUserIsRootDrive = False
		End If
	Next
	
End Function

'---- Sub CommandLineParser -----------------------------------------

Sub CommandLineParser

	If not IsWin2000 Then
		If (Wscript.Arguments.Count < 3) Then
			wscript.echo "Usage:"
			wscript.echo "------" & vbCRLF
 			WScript.Echo "NTFS Stuff CMDLine requires at least 3 arguments" & vbCrLF
			wscript.echo "cscript " & WScript.ScriptName & " [RootFolderPath]" & " [VerbosityLevel]" & " [RecursionLevel] /v" & vbCRLF
			wscript.echo "   - [RootFolderPath] : *required* full path to the file server's root folder"
			wscript.echo "   - [VerbosityLevel] : *required* 0 (normal) or 1 (high)"
			wscript.echo "   - [RecursionLevel] : *required* 0 (entire tree) or any other numeric value"
			wscript.echo "   - /v               : *optional* display progress in the command prompt window" & vbCrLF
			wscript.echo "Example 1: cscript " & WScript.ScriptName & " " & chr(34) & "c:\Data" & chr(34) & " 0 0" & vbCrLF
			wscript.echo "Example 2: cscript " & WScript.ScriptName & " " & chr(34) & "c:\Data" & chr(34) & " 0 0 /v" & vbCrLF
			Wscript.Quit(1)

		Else
			FolderByUser = Wscript.Arguments(0)

			If FolderByUserIsRootDrive(FolderByUser) then
				If Right(FolderByUser,1) <> "\" then
					FolderByUser = FolderByUser & "\"
				End If
			Else
				If Right(FolderByUser,1) = "\" then
					FolderByUser= Left(FolderByUser, Len(FolderByUser)-1)
				End If
			End If

			If ObjFSO.FolderExists(FolderByUser) = False Then
				Wscript.Echo "Invalid RootFolderPath: the specified path does not exist" & vbCrLF
				Wscript.Quit(1)
			End If
			
			SleepTime = 0
			Call SetLowPriorityProc

			Verbosity = Wscript.Arguments(1)
			
			Select Case Verbosity
				Case "0"
					Verbositylevel = 0
				Case "1"
					Verbositylevel = 1
				Case Else
					Wscript.Echo "Invalid VerbosityLevel: it must be equal to 0 or 1" & vbCrLF
					Wscript.Quit(1)
			End Select

			Recursion = Wscript.Arguments(2)

			If IsNumeric(Recursion) Then
				If (Recursion < 0) Then
					Wscript.echo "Invalid RecursionLevel: it must be equal or greater than 0" & vbCrLF
					Wscript.Quit(1)
				Else
					RecursionLevel = CInt(Trim(Recursion))
				End If
			Else
				Wscript.echo "Invalid RecursionLevel: it must be equal or greater than 0" & vbCrLF
				Wscript.Quit(1)	
			End If

			'Parsing optional arguments
			For i = 0 To CInt(Wscript.Arguments.Count) - 1
				If Trim(Lcase(Wscript.Arguments(i))) = "/v" Then
					boolV = True
				Else
					boolV = False
				End If
			Next
		End If

	Else
		If Wscript.Arguments.Count < 4 Then
			wscript.echo "Usage:"
			wscript.echo "------" & vbCRLF
			WScript.echo "NTFS Stuff CMDLine requires at least 4 arguments" & vbCrLF
			wscript.echo "cscript " & WScript.ScriptName & " [RootFolderPath]" & " [PriorityLevel]" & " [VerbosityLevel]" & " [RecursionLevel] /v" & vbCRLF
			wscript.echo "   - [RootFolderPath] : *required* full path to the file server's root folder"
			wscript.echo "   - [PriorityLevel]  : *required* 0 (low), 1 (normal) or 2 (high)"
			wscript.echo "   - [VerbosityLevel] : *required* 0 (normal) or 1 (high)"
			wscript.echo "   - [RecursionLevel] : *required* 0 (entire tree) or any other numeric value"
			wscript.echo "   - /v               : *optional* display progress in the command prompt window" & vbCrLF
			wscript.echo "Example 1: cscript " & WScript.ScriptName & " " & chr(34) & "c:\Data" & chr(34) & " 1 0 0" & vbCrLF
			wscript.echo "Example 2: cscript " & WScript.ScriptName & " " & chr(34) & "c:\Data" & chr(34) & " 1 0 0 /v" & vbCrLF
			wscript.Quit(1)

		Else
			FolderByUser = Wscript.Arguments(0)

			If FolderByUserIsRootDrive(FolderByUser) then
				If Right(FolderByUser,1) <> "\" then
					FolderByUser = FolderByUser & "\"
				End If
			Else
				If Right(FolderByUser,1) = "\" then
					FolderByUser= Left(FolderByUser, Len(FolderByUser)-1)
				End If
			End If

			If ObjFSO.FolderExists(FolderByUser) = False Then
				Wscript.Echo "Invalid RootFolderPath: the specified path does not exist" & vbCrLF
				Wscript.Quit(1)
			End If

			Priority = Wscript.Arguments(1)

			Select Case Priority
				Case "0"
					SleepTime = 400
				Case "1"
					Sleeptime = 150
				Case "2"
					Sleeptime = 0
				Case Else
					Wscript.echo "Invalid PriorityLevel: it must be equal to 0, 1 or 2" & vbCrLF
					Wscript.Quit(1)
				End Select
			
			Verbosity = Wscript.Arguments(2)
			
			Select Case Verbosity
				Case "0"
					Verbositylevel = 0
				Case "1"
					Verbositylevel = 1
				Case Else
					Wscript.Echo "Invalid VerbosityLevel: it must be equal to 0 or 1" & vbCrLF
					Wscript.Quit(1)
			End Select

			Recursion = Wscript.Arguments(3)

			If IsNumeric(Recursion) Then
				If (Recursion < 0) Then
					Wscript.echo "Invalid RecursionLevel: it must be equal or greater than 0" & vbCrLF
					Wscript.Quit(1)
				Else
					RecursionLevel = CInt(Trim(Recursion))
				End If
			Else
				Wscript.echo "Invalid RecursionLevel: it must be equal or greater than 0" & vbCrLF
				Wscript.Quit(1)	
			End If
			
			'Parsing optional arguments
			For i = 0 To CInt(Wscript.Arguments.Count) - 1
				If Trim(Lcase(Wscript.Arguments(i))) = "/v" Then
					boolV = True
				Else
					boolV = False
				End If
			Next
		End If

	End If

End Sub

'---- Function UACEnabled -------------------------------------------

Function UACEnabled

	Const HKEY_LOCAL_MACHINE = &H80000002

	Dim objRegistry, strKeyPath, strValue, strValueName
	Set objRegistry = GetObject("winmgmts:\\.\root\default:StdRegProv")
 
	strKeyPath = "SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\System"
	strValueName = "EnableLUA"
	objRegistry.GetDwordValue HKEY_LOCAL_MACHINE,strKeyPath,strValueName,strValue
	
	If (IsNull(strValue)) OR (strValue = "0") Then
		UACEnabled = 0
	Else
		UACEnabled = 1
	End If
	
End Function